// thread.h - Apstraktna klasa za nite (Thread).

#ifndef __thread__h__
#define __thread__h__
#include <wtypes.h>
#include <winbase.h>
#include <process.h>

class Thread {
public:
  enum State { READY, RUNNING, TERMINATING, TERMINATED, KILLED };
private:
  HANDLE rucka;
  State stanje;
  virtual void run () = 0;
  static void radi (LPVOID nit) {
    ((Thread *)nit)->run ();
    ((Thread *)nit)->exit ();
  }
protected:
  BOOL terminate () { return stanje == TERMINATING; }
  void exit () { stanje = TERMINATED; _endthread (); }
public:
  Thread (BOOL kreci=FALSE) { rucka = NULL; stanje = READY; }
  void start () {
    if (stanje == READY) {
      stanje = RUNNING;
      if ((rucka = (HANDLE)_beginthread (radi, 0, this)) == 0)
        throw GetLastError ();
    }
  }
  Thread &suspend () {
    if (stanje == RUNNING) {
      if (SuspendThread (rucka) == 0xffffffff) throw GetLastError ();
    }
    return *this;
  }
  Thread &resume () {
    if (stanje == RUNNING) {
      if (ResumeThread (rucka) == 0xffffffff) throw GetLastError ();
    }
    return *this;
  }
  static void sleep (DWORD vreme) { Sleep (vreme); }
  Thread &stop () {
    if (stanje == RUNNING) stanje = TERMINATING;
    return *this;
  }
  void kill () {
    if (stanje == RUNNING || stanje == TERMINATING) {
      if (! TerminateThread (rucka, 0)) throw GetLastError ();
      stanje = KILLED;
    }
  }
  BOOL join (DWORD _vreme=INFINITE) {
    if (stanje == RUNNING || stanje == TERMINATING) {
      DWORD uspeh = WaitForSingleObject (rucka, _vreme);
      if (uspeh == WAIT_FAILED) throw GetLastError ();
      return uspeh != WAIT_TIMEOUT;
    } else return TRUE;
  }
  State state () { return stanje; }
  virtual ~Thread () { stop (); join(); CloseHandle (rucka); }
};

#endif;
