// stranica.h - Deklaracije podataka i funkcija za stranicnu 
//              organizaciju memorije.

#ifndef __stranica__h__
#define __stranica__h__

typedef unsigned long Ulong;

class Proces2;

class Stranica {

  static Ulong    kapacitet;     // Kapacitet memorije [blok].
  static Ulong   *blokovi;       // Indikatori slobodnih blokova.
  static Proces2 *proces;        // Podaci za realizaciju aproksimaciju.
  static Ulong    stranica;      //   LRU smene stramica.
  static Ulong   *adresa;        // Logicka adresa str. prekida.

public:

  struct Ulaz {                  // ULAZ U STRANICNU TABLICU:
    unsigned prisutna:1;         // Stranica je u memoriji.
    unsigned koriscena:1;        // Stranica je koriscena.
    unsigned menjana:1;          // Stranica je menjana.
    unsigned blokirana:1;        // Ne sme da se izbaci iz memorije.
    unsigned cita:1,pise:1,radi:1;  // Mogucnosti za pristup.
    unsigned rezervisano:5;      //
    unsigned blok:16;
//    Ulong    blok:20;            // Broj bloka u memoriji.
  } ;

  enum { DUZ_POMERAJA = 10        // Broj bitova pomeraja.
       , VEL_STRANICE = 1 << 10   // Velicina stranice (1KB).
       , ZA_OS        = 4         // Rezervisano za OS (4*32=128KB).
  };

  static Ulong odredi_kapacitet ();      // Kapacitet memorije (ASM).
  static void  inicijalizacija ();       // Inic. modula za upravlj. mem.
  static Ulong trazi_blok ();            // Dodela slobodnog bloka.
  static void  vrati_blok (Ulong blok);  // Oslobadjanje bloka.
  static Ulong dodeli_blok ();           // Dodela uz smenu stranica.
  static void  obradi_str_prekid ();     // Stranica nije u memoriji.
  static int   pravi_str_tablicu (Proces2 *proc);   // Dolazak procesa.
  static void  oslobodi_sve_stranice (); // Odlazak procesa.
} ;

#endif
