// stranica.cpp - Definicije podataka i funkcija za stranicnu
//                organizaciju memorije.

#include "stranica.h"
#include "proces2.h"
//#include <fstream.h>

Ulong    Stranica::kapacitet=0;  // Kapacitet memorije [blok].
Ulong   *Stranica::blokovi=0;    // Indikatori slobodnih blokova.
Proces2 *Stranica::proces=0;     // Podaci za realizaciju aproksimaciju.
Ulong    Stranica::stranica=0;   //   LRU smene stramica.
Ulong   *Stranica::adresa=0;     // Logicka adresa str. prekida.

void Stranica::inicijalizacija () { // Inic. modula za upr. memorijom.
  kapacitet = odredi_kapacitet () ;
  int n = kapacitet / 32;
  blokovi = new Ulong [n];
  for (int i=0; i<n; i++ ) blokovi[i] = (i<ZA_OS) ? 0 : 0xffffffff;
}

Ulong Stranica::trazi_blok () {  // Dodela slobodnog bloka.
  for (Ulong i=0; i<kapacitet && blokovi[i]==0; i++);
  if (i == kapacitet) return 0;
  Ulong k = blokovi[i];
  for (int j=0; k&1==0; k>>=1,j++) ;
  blokovi[i] &= ~ (1ul << j);
  return i << 5 + j;
}

void Stranica::vrati_blok (Ulong blok) { // Oslobadanje bloka.
  Ulong i = blok >> 5; int j = blok & 0x1f;
  blokovi[i] |= 1ul << j;
}

Ulong Stranica::dodeli_blok (void) { // Dodela bloka uz smenu stranica.
  Ulong blok = trazi_blok (); 
  while (blok == 0) {
    Ulaz *ulaz;
    if (proces == 0) { proces = (Proces2 *)Proces2::prvi; stranica = 0;}
    if (proces->str_tabl == 0)
      { proces = (Proces2 *)proces->naredni; stranica=0; continue; }
    while (++stranica < proces->str_duz               &&
           (ulaz=proces->str_tabl+stranica)->prisutna &&
           (ulaz->blokirana || ulaz->koriscena)                  )
      ulaz->koriscena = 0;
    if (stranica == proces->str_duz)
      { proces = (Proces2 *)proces->naredni; stranica=0; continue; }

    proces->str_tabl[stranica].prisutna = 0;
    blok = ulaz->blok;
    if (ulaz->menjana) {
      FILE *dat = proces->str_datot;
      fseek (dat, (Ulong) stranica * VEL_STRANICE, SEEK_SET);
      union { Ulong bajt; char *adr; } ;
      bajt = (Ulong) blok * VEL_STRANICE;
      fwrite (adr, VEL_STRANICE, 1, dat);
    }
  }
  return blok;
}

void  Stranica::obradi_str_prekid () { // Obrada stranicnog prekida.
  Ulong blok     = dodeli_blok ();
  Ulong stranica = (Ulong) adresa >> DUZ_POMERAJA;
  Proces2 *radi = (Proces2 *)Proces2::radi;
  Ulaz *ulaz = radi->str_tabl + stranica;
  FILE *dat  = radi->str_datot;
  fseek (dat, stranica << DUZ_POMERAJA, SEEK_SET);
  union { Ulong bajt; char *adr; } ;
  bajt = (Ulong) blok * VEL_STRANICE;
  fread (adr, VEL_STRANICE, 1, dat);
  ulaz->blok      = blok;
  ulaz->prisutna  = 1;
  ulaz->koriscena = ulaz->menjana = ulaz->blokirana = 0;
}

int Stranica::pravi_str_tablicu (Proces2 *proc) { 
                              // Formiranje stranicne tablice za novi proces.
  FILE *prog = fopen (proc->program, "rb");
  FILE *dat  = tmpfile ();              // Privremena datoteka.
  char bafer [VEL_STRANICE];
  Ulong duz = 0; int i; Ulaz *tabl;
  if (prog == 0) return 1; if (dat == 0) return 2;
  while (!feof (prog)) {
    fread  (bafer, 1, VEL_STRANICE, prog);
    fwrite (bafer, 1, VEL_STRANICE, dat);
    if (ferror (dat)) { fclose (prog); fclose (dat); return 3; }
    duz++;
  }
  fclose (prog);
  proc->str_datot  = dat;
  proc->str_duz   = duz;
  proc->str_tabl  = tabl = new Ulaz [duz];;
  if (tabl == 0) { fclose (dat); return 4; }
  for (i=0; i<duz; i++) {
    tabl[i].prisutna = 0;
    tabl[i].cita = tabl[i].pise = tabl[i].radi = 1;
  }
  return 0;
}

void  Stranica::oslobodi_sve_stranice () {
                            // Oslobadjanje meorije pri odlasku procesa.
  Ulong i;
  Proces2 *radi = (Proces2 *)Proces2::radi;
  for (i=0; i<radi->str_duz; i++) {
    if (radi->str_tabl[i].prisutna)
      vrati_blok (radi->str_tabl[i].blok);
  }
  delete [] radi->str_tabl; radi->str_tabl  = 0;
  fclose (radi->str_datot); radi->str_datot = 0;
}

